#include "server.h"

constexpr int32_t pingInterval = 10000;
constexpr uint32_t pongTimeout = 5000;
uint8_t disconnectTimeoutCount = 3;

ServerConnection::ServerConnection() { }

void ServerConnection::configure(ServerConfiguration configuration, ServerConnectionCallbacks *callbacks) {
    controller = callbacks;
    this->configuration = configuration;
}

void ServerConnection::connect() {
    if (webSocket.isConnected()) {
        return;
    }
    if (controller == NULL) {
        Serial.println("[ERROR] No callbacks set for server");
        return;
    }

    webSocket.beginSSL(configuration.url, configuration.port, configuration.path);

    std::function<void(WStype_t, uint8_t *, size_t)> f = [this](WStype_t type, uint8_t *payload, size_t length) {
        this->webSocketEventHandler(type, payload, length);
    };
    webSocket.onEvent(f);
    webSocket.setReconnectInterval(configuration.reconnectTime);
}

void ServerConnection::disconnect() {
    webSocket.disconnect();
}

void ServerConnection::loop() {
    webSocket.loop();
}

void ServerConnection::webSocketEventHandler(WStype_t type, uint8_t * payload, size_t length) {
switch(type) {
    case WStype_DISCONNECTED:
        Serial.println("[INFO] Socket disconnected.");
        break;
    case WStype_CONNECTED:
        webSocket.sendTXT(configuration.key);
        Serial.printf("[INFO] Socket connected to url: %s\n", payload);
        webSocket.enableHeartbeat(pingInterval, pongTimeout, disconnectTimeoutCount);
        break;
    case WStype_TEXT:
        controller->sendServerError(SesameEvent::TextReceived);
        break;
    case WStype_BIN:
        controller->handleServerMessage(payload, length);
        break;
    case WStype_PONG:
        break;
    case WStype_PING:
    case WStype_ERROR:
    case WStype_FRAGMENT_TEXT_START:
    case WStype_FRAGMENT_BIN_START:
    case WStype_FRAGMENT:
    case WStype_FRAGMENT_FIN:
        controller->sendServerError(SesameEvent::UnexpectedSocketEvent);
        break;
    }
}

void ServerConnection::sendResponse(uint8_t* buffer, uint16_t length) {
    webSocket.sendBIN(buffer, length);
}