#include "storage.h"
#include "message.h"
#include <EEPROM.h>

void Storage::configure() {
    EEPROM.begin(messageCounterSize * remoteDeviceCount);
}

bool Storage::isDeviceIdValid(uint8_t deviceId) {
    return deviceId < remoteDeviceCount;
}

bool Storage::isMessageCounterValid(uint32_t counter, uint8_t deviceId) {
    return counter >= getNextMessageCounter(deviceId);
}

void Storage::didUseMessageCounter(uint32_t counter, uint8_t deviceId) {
    // Store the next counter, so that resetting starts at 0
    setMessageCounter(counter+1, deviceId);
}

void Storage::setMessageCounter(uint32_t counter, uint8_t deviceId) {
    int offset = deviceId * messageCounterSize;
    EEPROM.write(offset + 0, (counter >> 24) & 0xFF);
    EEPROM.write(offset + 1, (counter >> 16) & 0xFF);
    EEPROM.write(offset + 2, (counter >>  8) & 0xFF);
    EEPROM.write(offset + 3,  counter        & 0xFF);
    EEPROM.commit();
}

uint32_t Storage::getNextMessageCounter(uint8_t deviceId) {
    int offset = deviceId * messageCounterSize;
    uint32_t counter = (uint32_t) EEPROM.read(offset + 0) << 24;
    counter         += (uint32_t) EEPROM.read(offset + 1) << 16;
    counter         += (uint32_t) EEPROM.read(offset + 2) << 8;
    counter         += (uint32_t) EEPROM.read(offset + 3);
    return counter;
}

void Storage::printMessageCounters() {
    Serial.print("[INFO] Next message numbers:");
    for (uint8_t i = 0; i < remoteDeviceCount; i += 1) {
        Serial.printf(" %u", getNextMessageCounter(i));
    }
    Serial.println("");
}

void Storage::resetMessageCounters() {
    for (uint8_t i = 0; i < remoteDeviceCount; i += 1) {
        setMessageCounter(0, i);
    }
    Serial.println("[WARN] Message counters reset");
}
